/**
 * Created by Administrator on 2017/7/20.
 */
var fis = require('fis3');
var fgp=require("./fgp.js");
var common = require("./common");
var fs =  require("fs");

/**
 * 设置配置
 * @param config
 * @param now_media
 */
exports.set_config = function(config,now_media){
    //设置插件属性
    exports.set_plugins_config(config,now_media);

    //设置项目属性
    exports.set_project_config(config,now_media);

    //导入依赖
    exports.set_requires(config,now_media);

    //导入打包配置
    exports.set_package(config,now_media);

    //导入发布配置
    exports.set_deploy(config,now_media);
}


/**
 * 设置插件属性
 * @param config
 * @param now_media
 */
exports.set_plugins_config = function(config,now_media){
    if(!config || !fis)return;
    //获取默认值
    var cc_media = config.media || {};
    var cc_plugin = config.plugins || {};
    var cc_default = fgp.plugins || {};
    cc_default = fis.util.clone(cc_default);



    //先将项目的plugins和默认的值填充,以项目的plugins为默认
    fis.util.map(cc_default,function(i,item){
        if(cc_plugin[i]) {
            cc_default[i] = fis.util.merge(cc_default[i], cc_plugin[i]);
        }
    });

    //再将plugins多出的配置，放到默认
    fis.util.map(cc_plugin,function(i,item){
        if(!cc_default[i]) {
            cc_default[i] = cc_plugin[i];
        }
    });


    //再按照media上面的plugins去覆盖
    if(now_media) {
        var cc_media_now = cc_media[now_media]
        if (cc_media_now && cc_media_now.plugins && fis.util.is(cc_media_now.plugins, "Object")) {
            fis.util.map(cc_media_now.plugins, function (i2, item2) {
                //如果默认对象上面有这个插件，则吧media覆盖进去
                if (cc_default.hasOwnProperty(i2)) {
                    cc_default[i2] = fis.util.merge(cc_default[i2], item2);
                }
            })
        }
    }

    //使用了seajs的时候，如果base为空，则使用domain
    if(cc_default.fgp_seajs.isuse && !cc_default.fgp_seajs.seaconfig.config.base){
        cc_default.fgp_seajs.seaconfig.config.base =cc_media_now.domain+"/";
    }

    //吧默的plugins 绑到 config里面
    fis.util.map(cc_default,function(i,item){
        fis.set(i,item);
    })

    //fgp_seajs开启的时候，添加一个config文件
    if(cc_default.fgp_seajs && cc_default.fgp_seajs.isuse && cc_default.fgp_seajs.src){

        //流程开始的时候，定义一个sea_config文件，并且将useHash 保存到_useHash并且设置为false
        fis.emitter.on("release:start",function(ret){
            var cc_file = fis.file(fis.project.getProjectPath(), cc_default.fgp_seajs.src);

            cc_file._useHash = cc_file.useHash;
            cc_file.useHash = false;
            cc_file.setContent("");
            if(!ret.src[cc_default.fgp_seajs.src]){
                ret.src[cc_default.fgp_seajs.src] = cc_file;
            }
        })
    }

    //fgp_seajs开启，并且fgp_widge开启的时候，要把项目中的widget名，写到配置上去
    if(cc_default.fgp_widget && cc_default.fgp_widget.isuse && cc_default.fgp_seajs && cc_default.fgp_seajs.isuse && cc_default.fgp_seajs.seaconfig){
        fis.emitter.on("release:start",function(ret){
            var _widgetdir = fis.project.getProjectPath() + fis.get("fgp_widget.dir");
            var files = fs.readdirSync(_widgetdir);
            if(files) {
                cc_default.fgp_seajs.seaconfig.config.alias = cc_default.fgp_seajs.seaconfig.config.alias || {};

                var alias = cc_default.fgp_seajs.seaconfig.config.alias;
                files.forEach(function (item) {
                    var itemsrc = _widgetdir + "/" + item;
                    if (fis.util.isDir(itemsrc)) {
                        js = fis.file.wrap(itemsrc + "/" + item + ".js");
                        //widget存在JS
                        if (js.exists()) {
                            alias[js.getId()] = js.getId();
                        }
                    }
                })
                fis.set("fgp_seajs.seaconfig.config.alias",alias);
            }
        })
    }


    //吧默的plugins 绑到 config里面
    fis.util.map(cc_default,function(i,item){
        fis.set(i,item);
    })



    //把当前media绑定到config
    fis.set("current_media",cc_media[now_media]);

    config.plugins = cc_default;

}


exports.load_server_config = function(opt) {
    var bool_optimizer = opt.optimizer || false;
    var bool_domain = opt.domain || false;
    var bool_useHash = opt.useHash || false;


    //配置规则
    fis.match("**",{
        domain:bool_domain,
        //useHash:bool_useHash
    });

    fis.match("::image",{
        useHash:bool_useHash,
        release:"/image/$&",
        useMap:true
    });

    fis.match(/^\/node_modules\/.*?$/i, {
        release: false
    });

    fis.match("README.md", {
        release: false
    })

    fis.match(/.*\.(html|jsp|tpl|vm|htm|asp|aspx|php)$/, {
        release: '/$&'
    })

    fis.match("**.json", {
        useCompile: false,
        useOptimizer: false,
        release: false
    })

    fis.match(/^\/(widget.*?)\/(.*?)\/demo\/.*?$/i,{
        release: false
    })


    //*************************** JS *******************************//
    fis.match("**.js", {
        useHash:bool_useHash,
        parser: [
            fis.plugin("gpjs"),
            fis.plugin("gpconst")
        ],
        postprocessor: [
            fis.plugin("gpseajswrapper"),
            fis.plugin("require-async")
        ],
        lint: fis.plugin("jshint",{
            camelcase: true,
            curly: true,
            eqeqeq: true,
            forin: true,
            immed: true,
            latedef: true,
            newcap: true,
            noarg: true,
            noempty: true,
            node: true
        }),
        optimizer: bool_optimizer && fis.plugin('uglify-js', {
            mangle: {
                //不要压缩require关键字，否则seajs会识别不了require
                except: ['require', 'seajs']
            }
        })
    })

    fis.match(/^\/(.*)\.(js)$/i, {
        isMod: true,
        id: '$1',
        release: '/$&',
        useDomain: true
    })

    fis.match(/^\/js\/(lib\/.*)\.(js)$/i, {
        isMod: false
    })

    fis.match(/^\/(page\/.*?)\.js$/i,{
        id: 'js/$1',
        release: '/js/$&'
    })
    fis.match(/^\/(page.*?\/(.*?)\/\2)\.js/i,{
        id: 'js/$1',
        release: '/js/$1.js',
        is_page_js:true
    })
    //判断是否WIDGET的JS【不是WIDGET目录里面的JS，而是WIDGET的主JS】
    fis.match(/^\/(widget.*?\/(.*?)\/\2)\.js/i,{
        is_widget_js:true
    })
    fis.match("fis-conf.js",{
        release:false
    })


    //*************************** css *******************************//
    var cc_css_optimizer = fis.plugin('clean-css', {
       // 'keepBreaks': true //保持一个规则一个换行
    })

    fis.match(/^(.*)mixin\[.less|.scss]$/i, {
        release: false,
    })

    fis.match("**.less", {
        parser: fis.plugin("less"),
        useDomain: true,
        rExt: "css",
        useHash:bool_useHash,
        optimizer:cc_css_optimizer,
        useSprite: true,
        postprocessor: fis.plugin("gppx2rem"),
    })

    fis.match("**.scss", {
        parser: fis.plugin("node-sass"),
        useDomain: true,
        rExt: "css",
        useHash:bool_useHash,
        optimizer:cc_css_optimizer,
        useSprite: true,
        postprocessor: fis.plugin("gppx2rem"),
    })

    fis.match("**.css", {
        useDomain: true,
        useHash:bool_useHash,
        optimizer:cc_css_optimizer,
        useSprite: true,
        postprocessor: fis.plugin("gppx2rem"),
    })

    fis.match(/^\/page\/.*?\.(css|less|scss)$/i,{
        release: '/css/$&'
    })



    //********************** HTML **********************/
    fis.match("**.html", {
        parser: [
            fis.plugin("gpwidget"),
            fis.plugin("gpconst")
        ],
        postprocessor: [
            fis.plugin("gppx2rem"),
            fis.plugin("require-async")
        ]
    })

    fis.match(/^\/(page.*?)\/(.*?)\/\2\.tpl.html$/i,{
        release: '/$1/$2.tpl.html',
        useMap:true
    })

    fis.match("**.handlebars", {
        isJsLike: true,
        release:false,
        parser: [
            fis.plugin("gphandlebars"),
            fis.plugin("gpconst")
        ]
    })


}

/**
 * 导入项目配置
 * @param config
 */
exports.set_project_config = function(config,now_media){
    var cc_media = config.media || {};
    if(now_media && cc_media[now_media]){
        cc_media = cc_media[now_media];
    }

    if(config.name) {
        fis.cli.name = config.name;
    }

    //设置项目编码
    config.charset = config.charset || "utf-8";
    fis.config.set('project.charset', config.charset);

    //设置忽略文件
    if(config.ignore) {
        var cc_ignore = fis.get("project.ignore") || [];
        fis.util.map(config.ignore,function(i,item){
            cc_ignore.push(item);
        })
        fis.set("project.ignore",cc_ignore);
    }

    //设置livereload端口
    config.livereload.port && fis.config.set('livereload.port', config.livereload.port);



    //设置作者
    if(config.author){
        fis.set("project.author",config.author || "");
    }

}


/**
 * 导入打包配置
 * @param config
 * @param now_media
 */
exports.set_package=function(config,now_media){
    config = config || {};
    //打包
    if(config.package){
        //整理打包配置,默认是config.package,然后用now_media里面的package
        var cc_media = config.media || {};
        if(now_media && cc_media[now_media] && cc_media[now_media]["package"]){
            config.package = fis.util.merge(config.package,fis.util.clone(cc_media[now_media]["package"]))
        }


        fis.util.map(config.package,function(package_file,files){
            if(files  && fis.util.is(files,"Array")){
                fis.util.map(files,function(index,file){

                    if(file){
                        file = common.get_reg(file);
                        if(file) {
                            fis.match(file, {
                                "packTo": package_file,
                                "packOrder":index
                            })
                        }
                    }
                })
            }
        })
    }
}

//设置依赖
exports.set_requires=function(config){
    config = config || {};
    //添加依赖
    if(config.require && fis.util.is(config.require,"Array")){
        fis.util.map(config.require,function(i,item){
            if(item && item.key && item.value && fis.util.is(item.value,"Array")){
                var cc_role = item.key;
                cc_role = common.get_reg(cc_role);
                if(cc_role) {
                    fis.util.map(item.value,function(k,require){
                        item.value[k] = common.get_reg(require)
                    })
                    fis.match(cc_role, {
                        requires: item.value
                    })
                }
            }
        })
    }
}

/**
 * 设置发布流程
 * @param config
 * @param now_media
 */
exports.set_deploy = function(config,now_media){
    config = config || {};
    var cc_deploy = {};
    var cc_media = {};
    if(now_media && config.media && config.media[now_media]){
        cc_media = config.media[now_media];
    }
    if(cc_media.deploy === false){
        return false;
    }

    //处理合并 a ,b b>a
    var cc_deal = function(a,b){

        //覆盖
        if(b.type == "cover"){
            //循环a,吧B没有的 丢进去A,接下来，再处理合并每个子项
            fis.util.map(b,function(i,item){
                if(i == "type")return;
                if(a[i]) {
                    b[i] = cc_deal(a[i], b[i]);
                }
            })
        }
        //继承
        else{
            //循环a,吧B没有的 丢进去A,接下来，再处理合并每个子项
            fis.util.map(a,function(i,item){
                if(i == "type")return;
                if(!b[i]){
                    b[i] = a[i];
                }
                else if(fis.util.is(a[i],"Object")){
                    b[i] = cc_deal(a[i],b[i]);
                }
            })
        }
        return b;
    }

    //吧media.deploy定义进去
    cc_deploy = cc_media.deploy || {};
    //media 和 config 合并
    if(fis.util.is(config.deploy,"Object")){
        cc_deploy = cc_deal(config.deploy,cc_deploy)

    }

    if(fis.util.is(fgp.deploy,"Object")){
        cc_deploy = cc_deal(fgp.deploy,cc_deploy)
    }
    var cc_default = [];
    if(cc_deploy.encoding){
        cc_default.push(fis.plugin('encoding',cc_deploy.encoding));
    }
    if(cc_deploy.gpdeliver){
        cc_default.push(fis.plugin('gpdeliver',cc_deploy.gpdeliver));
    }

    fis.match("**",{
        deploy:cc_default
    })


    //循环更新发布文件
    fis.util.map(cc_deploy,function(i,item){
        if(i == "type" || i == "gpdeliver" || i=="encoding")return;
        i = common.get_reg(i);
        var cc_ret = fis.util.clone(cc_default);
        if(fis.util.is(item,"Object")){
            fis.util.map(item,function(plugin,config){
                config.reg_release = i;
                config.no_default = true;
                cc_ret.push(fis.plugin(plugin,config));
            })
            fis.match(i,{
                deploy:cc_ret
            });
        }
    })

}
